--@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
-- Page 40
-- Simple exercises in creating "Sequential Dates"
--@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
GO
--=====================================================================================================================
--      All Dates from the start of the year 2000
--=====================================================================================================================
--===== Obviously named variables
DECLARE  @pStartDate  DATETIME = '2000' --Parameter for an iTVF (Tip... 4 digit year returns the first of the year)
        ,@pCutoffDate DATETIME =  GETDATE() --Parameter for an iTVF
        ,@Days        INT
;
--===== Create the sequential dates
 SELECT SomeDate = DATEADD(dd,t.N,@pStartDate)
   FROM dbo.fnTally(0,DATEDIFF(dd,@pStartDate,@pCutoffDate)) t --Tally starts at "0" this time
  ORDER BY t.N --Note that this doesn't show up in the execution plan.
;
GO
--=====================================================================================================================
--      Rolling XX month window of months including the current month (XX months total)
--      Starts XX-1 months ago because current month is included.
--      Note that DATEADD(mm,DATEDIFF(mm,0,GETDATE()),0) calulates the first of the current month.
--=====================================================================================================================
DECLARE @pMonths INT = 12 --Parameter for an iTVF (Example of 12 months including current month)
;
--===== Create the sequential dates counting backwards (-t.N) for the floating window length.
     -- Notice the placement of -t.N .
 SELECT MonthStartDate = DATEADD(mm,DATEDIFF(mm,0,GETDATE())-t.N,0)
   FROM dbo.fnTally(0,@pMonths-1) t --Count from 0 to XX-1 = XX counts
  ORDER BY t.N --Note that this doesn't show up in the execution plan.
  --ORDER BY -t.N           --Ascending order but causes an expensive sort.
  --ORDER BY MonthStartDate --Ascending order but causes an expensive sort.
;
GO
-----------------------------------------------------------------------------------------------------------------------
--***** AWESOME SORTING TRICK *****
DECLARE @pMonths INT = 12 --Parameter for an iTVF (Example of 12 months including current month)
;
--===== Same thing but sorted in ascending order WITHOUT an extra sort just by changing the formula
     -- instead of changing the sort!
     -- Notice the placement of -t.N .
 SELECT MonthStartDate = DATEADD(mm,DATEDIFF(mm,0,GETDATE())-@pMonths+t.N+1,0) --Note the change in the formula.
   FROM dbo.fnTally(0,@pMonths-1) t --Count from 0 to XX-1 = XX counts
  ORDER BY t.N --Note that this doesn't show up in the execution plan.
;
GO
--=====================================================================================================================
--      Rolling XX month window of month start and next month start dates including the current month (XX months total).
--      Starts XX-1 months ago because current month is included.
--      Note that DATEADD(mm,DATEDIFF(mm,0,GETDATE()),0) calulates the first of the current month.
--=====================================================================================================================
DECLARE @pMonths INT = 12 --Parameter for an iTVF (Example of 12 months including current month)
;
--===== Create the sequential dates counting backwards for the floating window length.
     -- Notice the placement of -t.N .
 SELECT  MonthStartDate     = DATEADD(mm,DATEDIFF(mm,0,GETDATE())-t.N  ,0)
        ,NextMonthStartDate = DATEADD(mm,DATEDIFF(mm,0,GETDATE())-t.N+1,0)
   FROM dbo.fnTally(0,@pMonths-1) t --Count from 0 to XX-1 = XX counts
  ORDER BY t.N --Note that this doesn't show up in the execution plan.
;
GO
--=====================================================================================================================
--      Generate all 8 hour shift start/end dates/times and the shift number for the given year.
--      Note that DATEADD(yy,DATEDIFF(yy, 0,GETDATE()),0) calulates the first of the current year.
--=====================================================================================================================
DECLARE @pYear DATETIME = '2020' --Parameter for an iTVF (Tip... 4 digit year returns the first of the year)
;
   WITH cteYearStart AS (SELECT YearStart = DATEADD(yy,DATEDIFF(yy,0,@pYear),0)) --"DRY" out the formula (run time constant)
 SELECT  Shift# = t.N%3+1 --Convert t.N to the number 1 thru 3 repeating.
        ,StartDateTime = DATEADD(hh,t.N*8  ,ys.YearStart)
        ,EndDateTime   = DATEADD(hh,t.N*8+8,ys.YearStart)
   FROM cteYearStart ys
  CROSS APPLY dbo.fnTally(0,3*(DATEDIFF(dd,ys.YearStart,DATEADD(yy,1,ys.YearStart)))-1) t --3 shifts per day in year.
  ORDER BY t.N --Note that this doesn't show up in the execution plan.
;
GO